/*
 * Copyright (c) 2008 Lukas Mejdrech
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup icmp
 *  @{
 */

/** @file
 *  ICMP module.
 */

#ifndef __NET_ICMP_H__
#define __NET_ICMP_H__

#include <atomic.h>
#include <fibril_sync.h>

#include "../../include/icmp_codes.h"

#include "../../structures/int_map.h"
#include "../../structures/generic_field.h"

#include "icmp_header.h"

/** Type definition of the ICMP global data.
 *  @see icmp_globals
 */
typedef struct icmp_globals	icmp_globals_t;

/** Timeout locks map.
 *  Maps message identifiers to timeout implementing atomic variables.
 *  Sending fibril waits for its associated atomic variable to be set nonzero.
 *  Receiving fibril sets the associated atomic variable with the return value.
 */
INT_MAP_DECLARE( time_locks, atomic_t );

/** Echo specific data field.
 *  Used for bundle modules.
 *  The bundle module gets an index to the assigned echo specific data while connecting.
 *  The index is used in the future semi-remote calls instead of the ICMP phone.
 */
GENERIC_FIELD_DECLARE( echo_data, icmp_echo_t );

/** ICMP global data.
 */
struct	icmp_globals{
	/** IP module phone.
	 */
	int				ip_phone;
	/** Reserved packet prefix length.
	 */
	size_t			prefix;
	/** Maximal packet content length.
	 */
	size_t			content;
	/** Reserved packet suffix length.
	 */
	size_t			suffix;
	/** Packet address length.
	 */
	size_t			addr_len;
	/** Networking module phone.
	 */
	int				net_phone;
	/** The last used identifier number.
	 */
	icmp_param_t	last_used_id;
	/** The budled modules assigned echo specific data.
	 */
	echo_data_t		echo_data;
	/** Echo timeout locks.
	 */
	time_locks_t	time_locks;
	/** Safety lock.
	 */
	fibril_rwlock_t	lock;
};

#endif

/** @}
 */

